 
// Miscellaneous common operation on range representation objects

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "globals.h"
#include "Procs.h"
#include "Range.h"


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `Pool' : Taken from Stroustrup.                               //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

inline int max(int x, int y) {return x > y ? x : y ; }

void Pool :: init(unsigned sz) {
  esize  = max(sizeof(Link*), sz) ;
  head   = 0 ;
  nalloc = 0 ;  // debug
}

void Pool :: grow() {
  const int overhead = 12 ;
  const int chunk_size = 8 * 1024 - overhead ;
  const int nelem = (chunk_size - esize) / esize ;

  char* start = new char [chunk_size] ;
  char* last = &start [(nelem - 1) * esize] ;

  for(char* p = start ; p < last ; p += esize)
    ((Link*) p)->next = (Link*) (p + esize) ;
  ((Link*) last)->next = 0 ;
  head = (Link*) start ;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// Initialization of individual memory pools.                          //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

void RangeInit() {
  (DimRangeRep :: pool).init(sizeof(DimRangeRep))                           ;
  (DimSubRangeRep :: pool).init(sizeof(DimSubRangeRep))                     ;
  (CollapsedRangeRep :: pool).init(sizeof(CollapsedRangeRep))               ;
  (CollapsedSubRangeRep :: pool).init(sizeof(CollapsedSubRangeRep))         ;
  (BlockRangeRep :: pool).init(sizeof(BlockRangeRep))                       ;
  (BlockSubRangeRep :: pool).init(sizeof(BlockSubRangeRep))                 ;
  (PackedBlockSubRangeRep :: pool).init(sizeof(PackedBlockSubRangeRep))     ;
  (ExtendedBlockRangeRep :: pool).init(sizeof(ExtendedBlockRangeRep))       ;
  (ExtendedBlockSubRangeRep :: pool).init(sizeof(ExtendedBlockSubRangeRep)) ;
  (CyclicRangeRep :: pool).init(sizeof(CyclicRangeRep))                     ;
  (CyclicSubRangeRep :: pool).init(sizeof(CyclicSubRangeRep))               ;
  (StepRangeRep :: pool).init(sizeof(StepRangeRep))                         ;
  (StepSubRangeRep :: pool).init(sizeof(StepSubRangeRep))                   ;
  (PackedStepSubRangeRep :: pool).init(sizeof(PackedStepSubRangeRep))       ;

  (StrideRep :: pool).init(sizeof(StrideRep))                               ;
  (PackedStrideRep :: pool).init(sizeof(PackedStrideRep))                   ;
  (ExtendedStrideRep :: pool).init(sizeof(ExtendedStrideRep))               ;
}

MEMORY_POOL_DEFN(StrideRep)

// ---------------------------------------------------
//
// Things to do:

